@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
+import typing
 
-def deep_merge(target, data):
+def deep_merge(target : dict, data : dict) -> None:
     for key2, value2 in data.items():
         value1 = target.get(key2, None)
         if isinstance(value1, dict) and isinstance(value2, dict):
@@ -9,7 +10,7 @@ def deep_merge(target, data):
             target[key2] = value2
 
 
-def select(data, path, default_value=None):
+def select(data : dict, path : str, default_value : typing.Any = None) -> dict:
     paths = path.split(".")
     for path in paths:
         if isinstance(data, dict) and path in data:
@@ -21,7 +22,7 @@ def select(data, path, default_value=None):
     return data
 
 
-def update(data, path, value):
+def update(data : dict, path : str, value : typing.Any) -> dict:
     paths = path.split(".")
     for index in range(0, len(paths) - 1):
         path = paths[index]
@@ -52,8 +53,9 @@ def update(data, path, value):
         data[int(path)] = value
     else:
         data[path] = value
+    return data
 
-def ignore_none_item(data):
+def ignore_none_item(data: dict) -> dict:
     result = {}
     for key, value in data.items():
         if value is None:
@@ -77,7 +79,7 @@ class Object(dict):
         result = super().__setattr__(name, value)
         return result
 
-def to_object(data):
+def to_object(data: dict) -> Object:
     result = Object()
     for key, value in data.items():
         if isinstance(value, dict):
@@ -86,19 +88,28 @@ def to_object(data):
         setattr(result, key, value)
     return result
 
-def change(object_instance, data_dict : dict, object_key : str, dict_key : str = None) -> bool:
+def change(object_instance : typing.Union[object, dict], data_dict : typing.Union[object, dict], object_key : str, dict_key : str = None) -> bool:
     """Update property value of object_instance, using the value from data_dict. If value changed, return True. If value is equals, return False.
     """
     dict_key = dict_key or object_key
-    object_value = getattr(object_instance, object_key, None)
-    dict_value = data_dict.get(dict_key, None)
+    if isinstance(object_instance, dict):
+        object_value = object_instance.get(object_key, None)
+    else:
+        object_value = getattr(object_instance, object_key, None)
+    if isinstance(data_dict, dict):
+        dict_value = data_dict.get(dict_key, None)
+    else:
+        dict_value = getattr(data_dict, dict_key, None)
     if object_value == dict_value:
         return False
     else:
-        setattr(object_instance, object_key, dict_value)
+        if isinstance(object_instance, dict):
+            object_instance[object_key] = dict_value
+        else:
+            setattr(object_instance, object_key, dict_value)
         return True
 
-def changes(object_instance, data_dict, keys) -> bool:
+def changes(object_instance : typing.Union[object, dict], data_dict : typing.Union[object, dict], keys : typing.List[typing.Union[str, typing.Tuple[str, str]]]) -> bool:
     """Update property values of object_instance, using the value form data_dict. If any property changed, return True. If values are equal, return False. keys is a list of string or string pair.
     """
     result = False
