@@ -42,13 +42,12 @@ def unique(thelist):
             result.append(element)
     return result
 
-def replace(thelist, map, inplace=True):
+def replace(thelist, map, inplace=False):
     """Replace element in thelist, the map is collection of {old_value: new_value}.
     
     inplace == True, will effect the original list.
     inplace == False, will create a new list to return.
     """
-    thelist = list(thelist)
     if inplace:
         for index in range(len(thelist)):
             value = thelist[index]
@@ -57,6 +56,7 @@ def replace(thelist, map, inplace=True):
         return thelist
     else:
         newlist = []
+        thelist = list(thelist)
         for index in range(len(thelist)):
             value = thelist[index]
             if value in map:
@@ -83,3 +83,15 @@ def group(thelist):
             info[x] = 1
     return info
 
+
+def compare(old_set, new_set):
+    """Compare old_set and new_set, return set_new, set_delete, set_update.
+    """
+    old_set = set(old_set)
+    new_set = set(new_set)
+
+    set_new = new_set - old_set
+    set_delete = old_set - new_set
+    set_update = old_set.intersection(new_set)
+
+    return set_new, set_delete, set_update
