@@ -1,4 +1,4 @@
-# -*- coding: utf-8 -*-
+
 import functools
 import random
 import string
@@ -8,6 +8,7 @@ import base64
 import bizerror
 import typing
 import math
+import re
 from decimal import Decimal
 
 
@@ -24,11 +25,9 @@ YI = WAN * WAN
 default_encodings = ["utf8", "gb18030"]
 
 
+
 def random_string(length, choices=string.ascii_letters):
-    text = ""
-    for _ in range(0, length):
-        text += random.choice(choices)
-    return text
+    return "".join(random.choices(choices, k=length))
 
 def char_force_to_int(value):
     if value is None:
@@ -112,7 +111,6 @@ def wholestrip(text):
         text = text.replace(space, "")
     return text
 
-
 def split(text, seps, strip=False):
     """seps is a list of string, all sep in the seps are treated as delimiter.
     """
@@ -131,7 +129,6 @@ def split(text, seps, strip=False):
         results = row
     return results
 
-
 def str_composed_by(text, choices):
     """Test if text is composed by chars in the choices.
     """
@@ -142,7 +139,6 @@ def str_composed_by(text, choices):
 
 is_str_composed_by_the_choices = str_composed_by
 
-
 def is_hex_digits(text):
     """Test if all chars in text is hex digits.
     """
@@ -155,10 +151,9 @@ def join_lines(text):
     """
     if isinstance(text, str):
         return "".join(text.splitlines())
-    elif isinstance(text, bytes):
+    else:
         return b"".join(text.splitlines())
 
-
 def is_urlsafeb64_decodable(text):
     """Test if the text can be decoded by urlsafeb64 method.
     """
@@ -167,8 +162,8 @@ def is_urlsafeb64_decodable(text):
         return False
     if len(text) % 4 != 0:
         return False
-    return str_composed_by(join_lines(text), URLSAFEB64_CHARS)
-
+    text = join_lines(text)
+    return str_composed_by(text, URLSAFEB64_CHARS)
 
 def is_base64_decodable(text):
     """Test  if the text can be decoded by base64 method.
@@ -178,8 +173,8 @@ def is_base64_decodable(text):
         return False
     if len(text) % 4 != 0:
         return False
-    return str_composed_by(join_lines(text), BASE64_CHARS)
-
+    text = join_lines(text)
+    return str_composed_by(text, BASE64_CHARS)
 
 def is_unhexlifiable(text):
     """Test if the text can be decoded by unhexlify method.
@@ -191,7 +186,6 @@ def is_unhexlifiable(text):
         return False
     return str_composed_by(text, HEXLIFY_CHARS)
 
-
 def text_display_length(text, unicode_display_length=2, encoding=None):
     """Get text display length.
     """
@@ -231,7 +225,7 @@ def text_display_shorten(text, max_length, unicode_display_length=2, suffix="...
     result += suffix
     return result
 
-def smart_get_binary_data(text: typing.Union[str, bytes]) -> bytes:
+def smart_get_binary_data(text):
     if isinstance(text, str):
         if is_unhexlifiable(text):
             return binascii.unhexlify(text)
@@ -246,7 +240,7 @@ def smart_get_binary_data(text: typing.Union[str, bytes]) -> bytes:
     else:
         raise bizerror.NotSupportedTypeToCast()
 
-def is_chinese_character(c: str) -> bool:
+def is_chinese_character(c):
     """
     Block                                   Range       Comment
     CJK Unified Ideographs                  4E00-9FFF   Common
@@ -277,12 +271,12 @@ def is_chinese_character(c: str) -> bool:
         return True
     return False
 
-def binarify(data: typing.Union[str, bytes]) -> str:
+def binarify(data):
     if isinstance(data, str):
         data = data.encode("utf-8")
     return "".join(["{:08b}".format(x) for x in data])
 
-def unbinarify(text: typing.Union[str, bytes]) -> bytes:
+def unbinarify(text):
     if not text:
         return b""
     if isinstance(text, bytes):
@@ -290,7 +284,7 @@ def unbinarify(text: typing.Union[str, bytes]) -> bytes:
     from .listutils import chunk
     return bytes([int(x, 2) for x in chunk(text, 8)])
 
-def ints2bytes(ints: typing.List[int]) -> bytes:
+def ints2bytes(ints):
     result = b""
     for value in ints:
         if value:
@@ -299,7 +293,7 @@ def ints2bytes(ints: typing.List[int]) -> bytes:
             result += b'\x00'
     return result
 
-def int2bytes(value: int):
+def int2bytes(value):
     bs = []
     while value:
         bs.append(value % 256)
@@ -359,7 +353,6 @@ def combinations(values, length):
             result_words.add(subword)
     return result_words
 
-
 def captital_number(value, yuan="圆", digits="零壹贰叁肆伍陆柒捌玖", places="拾佰仟万亿", negative="负", float_places="角分厘毫丝忽微"):
     from .numericutils import float_split
 
@@ -464,3 +457,52 @@ def camel(value, **kwargs):
         return "".join([clean(x, clean_chars) for x in words])
     else:
         return "".join(words)
+
+
+
+def no_mapping(value):
+    return value
+
+def none_to_empty_string(value):
+    if value is None:
+        return ""
+    else:
+        return value
+
+def strip_string(value):
+    if isinstance(value, str):
+        return value.strip()
+    else:
+        return value
+
+
+def format_with_mapping(template, mapping, *args, **kwargs):
+    def trans(value):
+        if callable(mapping):
+            return mapping(value)
+        else:
+            return mapping.get(value, value)
+    new_args = []
+    new_kwargs = {}
+    names = re.findall('{([^}:]*)', template)
+    counter = 0
+    for index in range(len(names)):
+        if names[index] == "":
+            names[index] = counter
+            counter += 1
+        elif names[index].isdigit():
+            names[index] = int(names[index])
+    ps_args = [x for x in names if isinstance(x, int)]
+    if ps_args:
+        max_args_index = max(ps_args)
+        for index in range(max_args_index+1):
+            if index in names:
+                value = trans(args[index])
+            else:
+                value = None
+            new_args.append(value)
+    for key, value in kwargs.items():
+        if key in names:
+            value = trans(kwargs[key])
+        new_kwargs[key] = value
+    return template.format(*new_args, **new_kwargs)
