@@ -6,6 +6,11 @@ import logging
 logger = logging.getLogger(__name__)
 
 
+if not hasattr(threading, "get_ident"):
+    threading.get_ident = threading._get_ident
+
+get_ident = threading.get_ident
+
 class ServiceCore(object):
 
     def __init__(self):
